@@ -12,6 +12,7 @@
 
 import datetime
 import debtcollector
+import functools
 import itertools
 import logging
 import logging.config
@@ -32,6 +33,15 @@ if six.PY3:
     from functools import reduce
 
 
+try:
+    # Test if to_primitive() has the fallback parameter added
+    # in oslo.serialization 2.21.1
+    jsonutils.to_primitive(1, fallback=repr)
+    _HAVE_JSONUTILS_FALLBACK = True
+except TypeError:
+    _HAVE_JSONUTILS_FALLBACK = False
+
+
 def _dictify_context(context):
     if getattr(context, 'get_logging_values', None):
         return context.get_logging_values()
@@ -138,17 +148,29 @@ def _get_error_summary(record):
     # If we have an exception, format it to be included in the
     # output.
     if exc_info:
-        # Build the exception summary in the line with the
-        # primary log message, to serve as a mnemonic for error
-        # and warning cases.
-        error_summary = traceback.format_exception_only(
-            exc_info[0],
-            exc_info[1],
-        )[0].rstrip()
-        # Remove the local reference to the exception and
-        # traceback to avoid a memory leak through the frame
-        # references.
-        del exc_info
+        try:
+            # Build the exception summary in the line with the
+            # primary log message, to serve as a mnemonic for error
+            # and warning cases.
+            error_summary = traceback.format_exception_only(
+                exc_info[0],
+                exc_info[1],
+            )[0].rstrip()
+            # If the exc_info wasn't explicitly passed to us, take only the
+            # first line of it. _Remote exceptions from oslo.messaging append
+            # the full traceback to the exception message, so we want to avoid
+            # outputting the traceback unless we've been passed exc_info
+            # directly (via LOG.exception(), for example).
+            if not record.exc_info:
+                error_summary = error_summary.split('\n', 1)[0]
+        except TypeError as type_err:
+            # Work around https://bugs.python.org/issue28603
+            error_summary = "<exception with %s>" % six.text_type(type_err)
+        finally:
+            # Remove the local reference to the exception and
+            # traceback to avoid a memory leak through the frame
+            # references.
+            del exc_info
 
     return error_summary
 
@@ -169,7 +191,12 @@ class JSONFormatter(logging.Formatter):
             self.hostname = None
 
     def formatException(self, ei, strip_newlines=True):
-        lines = traceback.format_exception(*ei)
+        try:
+            lines = traceback.format_exception(*ei)
+        except TypeError as type_error:
+            # Work around https://bugs.python.org/issue28603
+            msg = six.text_type(type_error)
+            lines = ['<Unprintable exception due to %s>\n' % msg]
         if strip_newlines:
             lines = [moves.filter(
                 lambda x: x,
@@ -211,18 +238,33 @@ class JSONFormatter(logging.Formatter):
         for key in getattr(record, 'extra_keys', []):
             if key not in extra:
                 extra[key] = getattr(record, key)
-        # If we saved a context object, explode it into the extra
-        # dictionary because the values are more useful than the
+        # The context object might have been given from the logging call. if
+        # that was the case, it'll come in the 'extra' entry already. If not,
+        # lets use the context we fetched above. In either case, we explode it
+        # into the 'context' entry because the values are more useful than the
         # object reference.
-        if 'context' in extra:
-            extra.update(_dictify_context(context))
-            del extra['context']
+        if 'context' in extra and extra['context']:
+            message['context'] = _dictify_context(extra['context'])
+        elif context:
+            message['context'] = _dictify_context(context)
+        else:
+            message['context'] = {}
+        extra.pop('context', None)
         message['extra'] = extra
 
         if record.exc_info:
             message['traceback'] = self.formatException(record.exc_info)
 
-        return jsonutils.dumps(message)
+        if _HAVE_JSONUTILS_FALLBACK:
+            # Bug #1593641: If an object cannot be serialized to JSON, convert
+            # it using repr() to prevent serialization errors. Using repr() is
+            # not ideal, but serialization errors are unexpected on logs,
+            # especially when the code using logs is not aware that the
+            # JSONFormatter will be used.
+            convert = functools.partial(jsonutils.to_primitive, fallback=repr)
+            return jsonutils.dumps(message, default=convert)
+        else:
+            return jsonutils.dumps(message)
 
 
 class FluentFormatter(logging.Formatter):
@@ -245,7 +287,12 @@ class FluentFormatter(logging.Formatter):
             self.hostname = None
 
     def formatException(self, exc_info, strip_newlines=True):
-        lines = traceback.format_exception(*exc_info)
+        try:
+            lines = traceback.format_exception(*exc_info)
+        except TypeError as type_error:
+            # Work around https://bugs.python.org/issue28603
+            msg = six.text_type(type_error)
+            lines = ['<Unprintable exception due to %s>\n' % msg]
         if strip_newlines:
             lines = reduce(lambda a, line: a + line.rstrip().splitlines(),
                            lines, [])
@@ -275,12 +322,18 @@ class FluentFormatter(logging.Formatter):
         for key in getattr(record, 'extra_keys', []):
             if key not in extra:
                 extra[key] = getattr(record, key)
-        # If we saved a context object, explode it into the extra
-        # dictionary because the values are more useful than the
+        # The context object might have been given from the logging call. if
+        # that was the case, it'll come in the 'extra' entry already. If not,
+        # lets use the context we fetched above. In either case, we explode it
+        # into the extra dictionary because the values are more useful than the
         # object reference.
-        if 'context' in extra:
-            extra.update(_dictify_context(context))
-            del extra['context']
+        if 'context' in extra and extra['context']:
+            message['context'] = _dictify_context(extra['context'])
+        elif context:
+            message['context'] = _dictify_context(context)
+        else:
+            message['context'] = {}
+        extra.pop('context', None)
         message['extra'] = extra
 
         if record.exc_info:
@@ -450,11 +503,22 @@ class ContextFormatter(logging.Formatter):
     def formatException(self, exc_info, record=None):
         """Format exception output with CONF.logging_exception_prefix."""
         if not record:
-            return logging.Formatter.formatException(self, exc_info)
+            try:
+                return logging.Formatter.formatException(self, exc_info)
+            except TypeError as type_error:
+                # Work around https://bugs.python.org/issue28603
+                msg = six.text_type(type_error)
+                return '<Unprintable exception due to %s>\n' % msg
 
         stringbuffer = moves.StringIO()
-        traceback.print_exception(exc_info[0], exc_info[1], exc_info[2],
-                                  None, stringbuffer)
+        try:
+            traceback.print_exception(exc_info[0], exc_info[1], exc_info[2],
+                                      None, stringbuffer)
+        except TypeError as type_error:
+            # Work around https://bugs.python.org/issue28603
+            msg = six.text_type(type_error)
+            stringbuffer.write('<Unprintable exception due to %s>\n' % msg)
+
         lines = stringbuffer.getvalue().split('\n')
         stringbuffer.close()
 
