@@ -415,19 +415,38 @@ class JSONFormatterTestCase(LogTestBase):
                                        formatter=formatters.JSONFormatter)
         self._set_log_level_with_cleanup(self.log, logging.DEBUG)
 
-    def test_json(self):
+    def test_json_w_context_in_extras(self):
         test_msg = 'This is a %(test)s line'
         test_data = {'test': 'log'}
         local_context = _fake_context()
         self.log.debug(test_msg, test_data, key='value', context=local_context)
+        self._validate_json_data('test_json_w_context_in_extras', test_msg,
+                                 test_data, local_context)
 
+    def test_json_w_fetched_global_context(self):
+        test_msg = 'This is a %(test)s line'
+        test_data = {'test': 'log'}
+        local_context = _fake_context()
+        # NOTE we're not passing the context explicitly here. But it'll add the
+        # context to the extras anyway since the call to fake_context adds the
+        # context to the thread. The context will be fetched with the
+        # _update_record_with_context call that's done in the formatter.
+        self.log.debug(test_msg, test_data, key='value')
+        self._validate_json_data('test_json_w_fetched_global_context',
+                                 test_msg, test_data, local_context)
+
+    def _validate_json_data(self, testname, test_msg, test_data, ctx):
         data = jsonutils.loads(self.stream.getvalue())
         self.assertTrue(data)
         self.assertIn('extra', data)
+        self.assertIn('context', data)
         extra = data['extra']
+        context = data['context']
+        self.assertNotIn('context', extra)
         self.assertEqual('value', extra['key'])
-        self.assertEqual(local_context.auth_token, extra['auth_token'])
-        self.assertEqual(local_context.user, extra['user'])
+        self.assertEqual(ctx.user, context['user'])
+        self.assertEqual(ctx.user_name, context['user_name'])
+        self.assertEqual(ctx.project_name, context['project_name'])
         self.assertEqual('test-json', data['name'])
 
         self.assertEqual(test_msg % test_data, data['message'])
@@ -435,7 +454,7 @@ class JSONFormatterTestCase(LogTestBase):
         self.assertEqual(test_data, data['args'])
 
         self.assertEqual('test_log.py', data['filename'])
-        self.assertEqual('test_json', data['funcname'])
+        self.assertEqual(testname, data['funcname'])
 
         self.assertEqual('DEBUG', data['levelname'])
         self.assertEqual(logging.DEBUG, data['levelno'])
@@ -522,6 +541,52 @@ class JSONFormatterTestCase(LogTestBase):
         self.assertIn('error_summary', data)
         self.assertEqual('', data['error_summary'])
 
+    def test_exception_without_exc_info_passed(self):
+        ctxt = _fake_context()
+        ctxt.request_id = six.text_type('99')
+        try:
+            raise RuntimeError('test_exception\ntraceback\nfrom\nremote error')
+        except RuntimeError:
+            self.log.warning('testing', context=ctxt)
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertIn('error_summary', data)
+        self.assertEqual('RuntimeError: test_exception', data['error_summary'])
+
+    def test_exception_with_exc_info_passed(self):
+        ctxt = _fake_context()
+        ctxt.request_id = six.text_type('99')
+        try:
+            raise RuntimeError('test_exception\ntraceback\nfrom\nremote error')
+        except RuntimeError:
+            self.log.exception('testing', context=ctxt)
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertIn('error_summary', data)
+        self.assertEqual('RuntimeError: test_exception'
+                         '\ntraceback\nfrom\nremote error',
+                         data['error_summary'])
+
+    def test_fallback(self):
+        if not formatters._HAVE_JSONUTILS_FALLBACK:
+            self.skipTest("need the fallback parameter of "
+                          "jsonutils.to_primitive() added in "
+                          "oslo_serialization 2.21.1")
+
+        class MyObject(object):
+            def __str__(self):
+                return 'str'
+
+            def __repr__(self):
+                return 'repr'
+
+        obj = MyObject()
+        self.log.debug('obj=%s', obj)
+
+        data = jsonutils.loads(self.stream.getvalue())
+        self.assertEqual('obj=str', data['message'])
+        # Bug #1593641: If an object of record.args cannot be serialized,
+        # convert it using repr() to prevent serialization error on logging.
+        self.assertEqual(['repr'], data['args'])
+
 
 def get_fake_datetime(retval):
     class FakeDateTime(datetime.datetime):
@@ -564,9 +629,9 @@ class FluentFormatterTestCase(LogTestBase):
         self.assertIn('lineno', data)
         self.assertIn('extra', data)
         extra = data['extra']
+        context = data['context']
         self.assertEqual('value', extra['key'])
-        self.assertEqual(local_context.auth_token, extra['auth_token'])
-        self.assertEqual(local_context.user, extra['user'])
+        self.assertEqual(local_context.user, context['user'])
         self.assertEqual('test-fluent', data['name'])
 
         self.assertEqual(test_msg % test_data, data['message'])
@@ -1496,6 +1561,7 @@ class LogConfigOptsTestCase(BaseTestCase):
                          self.CONF.log_date_format)
 
         self.assertEqual(False, self.CONF.use_syslog)
+        self.assertEqual(False, self.CONF.use_json)
 
     def test_log_file(self):
         log_file = '/some/path/foo-bar.log'
@@ -1553,6 +1619,15 @@ class LogConfigOptsTestCase(BaseTestCase):
             self.assertIsInstance(formatter,
                                   formatters.ContextFormatter)
 
+    def test_json_formatter(self):
+        self.CONF(['--use-json'])
+        log._setup_logging_from_conf(self.CONF, 'test', 'test')
+        logger = log._loggers[None].logger
+        for handler in logger.handlers:
+            formatter = handler.formatter
+            self.assertIsInstance(formatter,
+                                  formatters.JSONFormatter)
+
     def test_handlers_cleanup(self):
         """Test that all old handlers get removed from log_root."""
         old_handlers = [log.handlers.ColorHandler(),
